@@ -86,6 +86,7 @@ audio_alsa_sink::audio_alsa_sink (int sampling_freq,
     d_device_name (device_name.empty() ? default_device_name() : device_name),
     d_pcm_handle (0),
     d_hw_params ((snd_pcm_hw_params_t *)(new char[snd_pcm_hw_params_sizeof()])),
+    d_sw_params ((snd_pcm_sw_params_t *)(new char[snd_pcm_sw_params_sizeof()])),
     d_nperiods (nperiods <= 0 ? default_nperiods() : nperiods),
     d_period_time_us ((unsigned int) ((period_time <= 0 ?
 				       default_period_time() : period_time) * 1e6)),
@@ -240,6 +241,25 @@ audio_alsa_sink::check_topology (int ninputs, int noutputs)
     return false;
   }
 
+  // get current s/w params
+  err = snd_pcm_sw_params_current (d_pcm_handle, d_sw_params);
+  if (err < 0)
+    bail ("snd_pcm_sw_params_current", err);
+  
+  // Tell the PCM device to wait to start until we've filled
+  // it's buffers half way full.  This helps avoid audio underruns.
+
+  err = snd_pcm_sw_params_set_start_threshold(d_pcm_handle,
+					      d_sw_params,
+					      d_nperiods * d_period_size / 2);
+  if (err < 0)
+    bail ("snd_pcm_sw_params_set_start_threshold", err);
+
+  // store the s/w params
+  err = snd_pcm_sw_params (d_pcm_handle, d_sw_params);
+  if (err < 0)
+    bail ("snd_pcm_sw_params", err);
+
   d_buffer_size_bytes =
     d_period_size * nchan * snd_pcm_format_size (d_format, 1);
 
@@ -279,6 +299,7 @@ audio_alsa_sink::~audio_alsa_sink ()
 
   snd_pcm_close(d_pcm_handle);
   delete [] ((char *) d_hw_params);
+  delete [] ((char *) d_sw_params);
   delete [] d_buffer;
 }
 
