@@ -86,6 +86,7 @@ audio_alsa_source::audio_alsa_source (int sampling_freq,
     d_device_name (device_name.empty() ? default_device_name() : device_name),
     d_pcm_handle (0),
     d_hw_params ((snd_pcm_hw_params_t *)(new char[snd_pcm_hw_params_sizeof()])),
+    d_sw_params ((snd_pcm_sw_params_t *)(new char[snd_pcm_sw_params_sizeof()])),
     d_nperiods (nperiods <= 0 ? default_nperiods() : nperiods),
     d_period_time_us ((unsigned int) ((period_time <= 0 ?
 				       default_period_time() : period_time) * 1e6)),
@@ -211,7 +212,6 @@ audio_alsa_source::audio_alsa_source (int sampling_freq,
   set_output_multiple (d_period_size);
 }
 
-
 bool
 audio_alsa_source::check_topology (int ninputs, int noutputs)
 {
@@ -281,6 +281,7 @@ audio_alsa_source::~audio_alsa_source ()
 
   snd_pcm_close(d_pcm_handle);
   delete [] ((char *) d_hw_params);
+  delete [] ((char *) d_sw_params);
   delete [] d_buffer;
 }
 
